-------------------------------------------------------------------------------
-- Company    : ope
-- Author     : Olaf Petzold
--
-- Project    : Logic Analyzer
-- Title      : Testbench for design "ram_1536x16"
-- File       : tb_ram_1536x16.vhd
-- Standard   : VHDL'93
-- Create Date: 2005-10-02
-- Last update: 2005-10-04
-- Platform   : XS2S100 Xilinx
-- Description: 
--    This is part of the logic analyzer project. For
--    further informations see at http://www.mikrocontroller.net.
--   
--
-- Copyright  : Copyright (c) 2005 Olaf Petzold
-- Revision   : $Id: TB_ram_1536x16.vhd,v 1.1 2005/10/04 08:50:28 olaf Exp $
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use std.textio.all;
use work.la_pkg.all;


entity tb_ram_1536x16 is
end entity tb_ram_1536x16;


architecture behavioral of tb_ram_1536x16 is

   --for all: ram_1536x16 use entity work.ram_1536x16(banking);
   
   -- component generics
   constant PERIOD         : time := 10 ns;
   constant RESET_ACTIVE   : std_logic := '1';

   -- component ports
   signal clk        : std_logic := '0';
   signal reset      : std_logic;
   signal AB         : std_logic_vector(10 downto 0);
   signal DB         : std_logic_vector(15 downto 0);   
   signal DB_I       : std_logic_vector(15 downto 0);
   signal DB_O       : std_logic_vector(15 downto 0);
   signal we         : std_logic := '0';

   -- signals
   type enable_t is record
      prbs           : std_logic;
      addr_cntr      : std_logic;
      ram            : std_logic;
   end record enable_t;
   signal enable     : enable_t;
   
   -- tb variables
   signal clk_re_cnt : natural := 0;
   signal clk_fe_cnt : natural := 0;
	signal check_time	: natural;
	signal expected   : std_logic;
	signal start  	   : natural;

   -- test case types
   type test_mode_t is (
      reset_tests,
      idle_tests,
      write_tests,
      read_tests
   );
   signal test_mode	: test_mode_t := reset_tests;
   
begin

   write(output, string'("run this TB x00 ns!"));

   ----------------------------------------------------------------------------
   
   DUT: entity work.ram_1536x16(banking)
      generic map (
         RESET_ACTIVE => RESET_ACTIVE
      )
      port map (
         clk   => clk,
         reset => reset,
         en    => enable.ram,
         AB    => AB,
         DB_I  => DB_I,
         DB_O  => DB_O,
         we    => we
      );

   ----------------------------------------------------------------------------

   adress_counter: entity work.counter_g
      generic map (
         BIT_WIDTH    => AB'length,
         CLOCK_ENABLE => true,
         DOWN_COUNTER => false,
         LOADABLE     => false,
         RESET_ACTIVE => RESET_ACTIVE
      )
      port map (
         clk   => clk,
         reset => reset,
         en    => enable.addr_cntr,
         ld    => open,
         data  => open,
         count => AB
      );
   
   ----------------------------------------------------------------------------

	clock_proc: process (clk, reset, clk_re_cnt) is
	begin

      clk <= not clk after PERIOD/2;
      
      if reset = not RESET_ACTIVE then
         if rising_edge(clk) then
            clk_re_cnt <= clk_re_cnt + 1;
         end if;
      end if;

      clk_fe_cnt <= clk_re_cnt after PERIOD/2;

	end process;

   ----------------------------------------------------------------------------

   wavegen: block
      
      signal clk_lfsr : std_logic;
      
   begin

      clk_lfsr <= clk and enable.prbs;
      
      PRBS_gen: entity work.LFSR16
         generic map (
            RESET_ACTIVE => RESET_ACTIVE
         )
         port map (
            reset    => reset,
            clk	   => clk_lfsr,
            q		   => DB
         );  
   end block;
   
   
   ----------------------------------------------------------------------------

   ram_fsm: block is

      type state_t is (
         idle_state,
         read_state,
         write_state
      ); 
      signal state, next_state   : state_t;
      signal m_enable            : enable_t;
      signal m_we                : std_logic;
      
   begin  -- block ram_ctrl
      
      sync_proc: process (clk, reset)
      begin
         
         if (reset = RESET_ACTIVE) then
            state                <= idle_state;
            enable.addr_cntr     <= '0';
            enable.ram           <= '0';
            we                   <= '0';
         elsif rising_edge(clk) then
            state                <= next_state;
            enable.addr_cntr     <= m_enable.addr_cntr;
            enable.ram           <= m_enable.ram ;
            we                   <= m_we;
         end if;
         
      end process;
      
      -- MEALY State Machine - Outputs based on state and inputs
      output_decode: process (state)
      begin
         
         -- statements to decode internal output signals
         if (state = write_state) then
            m_enable.addr_cntr   <= '1';
            m_enable.ram         <= '1';
            m_we                 <= '1';  
         elsif (state = read_state) then
            m_enable.addr_cntr   <= '1';
            m_enable.ram         <= '1';
            m_we                 <= '0';
         else
            -- idle state
            m_enable.addr_cntr   <= '0';
            m_enable.ram         <= '0';
            m_we                 <= '0';
         end if;
         
      end process;
      
      next_state_decode: process (state, test_mode)
      begin
         
         -- declare default state for next_state to avoid latches
         next_state <= state;
         
         -- statements to decode next_state
         case (state) is
            when idle_state =>
               if (test_mode = read_tests) then
                  next_state <= read_state;
               elsif (test_mode = write_tests) then
                  next_state <= write_state;
               end if;
            when write_state =>
               if (test_mode = idle_tests) then
                  next_state <= idle_state;
               end if;
            when read_state =>
               if (test_mode = idle_tests) then
                  next_state <= idle_state;
               end if;
         end case;
         
      end process;

   end block ram_fsm;
   
   ----------------------------------------------------------------------------

   TB: block is
   begin
      
      test_select: process (clk_re_cnt, test_mode) is
         
         constant TOP_ADDR : natural := 1536;
         
         constant t0 : natural := 1;
         constant t1 : natural := t0 + 7;
         constant t2 : natural := t1 + TOP_ADDR;
         constant t3 : natural := t2 + 5;
         constant t4 : natural := t3 + 5;
         constant t5 : natural := t4 + 10;
         constant t6 : natural := t5 + TOP_ADDR;
         
      begin  -- process test_select
         
         case clk_re_cnt is
            when t0     => test_mode <= idle_tests;
            when t1     => test_mode <= write_tests;
            when t2     => test_mode <= idle_tests;
            when t3     => test_mode <= reset_tests;
            when t4     => test_mode <= idle_tests;
            when t5     => test_mode <= read_tests;
            when t6     => test_mode <= idle_tests;                                                   
            when others => null;
         end case;

         -- start is equal at time where tests are changed
         if test_mode'event then
            start <= clk_re_cnt;   
         end if;

      end process;

      
      tb_ctrl: process (test_mode) is
      begin
         
         if test_mode = reset_tests then
            enable.prbs    <= '0';
            reset          <= '1', '0' after PERIOD;
         elsif test_mode = idle_tests then
            enable.prbs    <= '0' after 5*PERIOD;
         elsif test_mode = write_tests then
            enable.prbs    <= '1' after 2*PERIOD;
         elsif test_mode = read_tests then
            -- that's wrong, BUG! Testing only!
            -- prbs starts after 2 clk cycles with enable.prbs = '1'
            -- ONLY for test purpose here. Follwing timing diagramm
            -- (http://www.xilinx.com/bvdocs/publications/ds001.pdf pg. 25)
            -- the data will be after Tbcko on DB_O!!
            enable.prbs    <= '1' after 2*PERIOD;
         end if;

      end process;

      DB_I           <= DB;
      
   end block TB;

   -------------------------------------------------------------------------------
   
--   assert_proc: process (clk, clk_re_cnt, check_time, match, expected) is
--   begin

--      -- check on falling edge
--      if (clk_re_cnt = check_time) and (clk_re_cnt /= 0) then
--         if falling_edge(clk) then
--            write(output, string'("check @" & natural'image(clk_re_cnt) & ", "));
--            assert match = expected
--               report "E@TB: trigger condition." severity error;
--         end if;
--      end if;

--   end process;
   
end architecture;

-------------------------------------------------------------------------------

-- WON'T COMPILE
--configuration tb_ram_1536x16_cfg of tb_ram_1536x16 is
--   -- block/arch conf
--   for behavioral
--      -- components conf
--      for DUT : ram_1536x16
--         use entity work.ram_1536x16(infering);
--      end for;
--   end for;
--end tb_ram_1536x16_cfg;

