-------------------------------------------------------------------------------
-- Company    : ope
-- Author     : Olaf Petzold
--
-- Project    : Logic Analyzer
-- Title      : counter_g
-- File       : counter_g.vhd
-- Standard   : VHDL'93
-- Create Date: 2005-08-04
-- Last update: 2005-09-30
-- Platform   : generic
-- Description: 
--    This is part of the logic analyzer project. For
--    further informations see at http://www.mikrocontroller.net.
--   
--		A Generic Counter
--
--    Generic Options:
--			BIT_WIDTH: defines the structure of counter
--					8bit default
--			CLOCK_ENABLE: defines the structure of counter  at synthese
--					attachs a ClockEnable pin (default)
--			DOWN_COUNTER: controls the count direction on an up/down counter.
--					true : up counter (default)
--					false: down counter
--    	LOADABLE: designs two inputs more, an input <ld> to load the preload
--             data from <data> which does have the same bit width as the
--             counter self.
--    	RESET_ACTIVE: determines the reset activity, '0' or '1'.
--
--		Port Description:
--			clk: clock
--			reset: asynchronous reset, depends on RESET_ACTIVE
--			en: clock enable, high active; availability depends on CLOCK_ENABLE
--			ld: data load with clock's rising edge; availability depends on LOADABLE
--			data: preload data; availability depends on LOADABLE
--			count: counter output
--
-- Copyright  : Copyright (c) 2005 Olaf Petzold
-- Revision   : $Id: counter_g.vhd,v 1.2 2005/09/30 21:31:35 olaf Exp $
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity counter_g is
	generic(
		BIT_WIDTH	:	positive	:= 8;
		CLOCK_ENABLE:	boolean	:= true;
		DOWN_COUNTER:	boolean	:= false;
		LOADABLE		:	boolean	:= false;
      -- to be implemented
		-- POS_EDGE:   boolean := true; -- design triggers on rising_edge of clock
		RESET_ACTIVE:	std_logic := '1'
	);
	port(
		clk		: in  std_logic;				-- rising edge trigger count
		reset		: in  std_logic;				-- asynchronous reset
		en			: in  std_logic := '0';		-- clock enable, high active
		ld			: in  std_logic := '0';		-- data load with clock's rising edge 
		data		: in  std_logic_vector(BIT_WIDTH-1 downto 0) := (others => '0');
		count		: out std_logic_vector(BIT_WIDTH-1 downto 0)
	);
end counter_g;


architecture behavioral of counter_g is

	signal m_count : std_logic_vector(BIT_WIDTH-1 downto 0);

begin
   
	process (clk, reset) is
	begin

      if reset = RESET_ACTIVE then
			m_count <= (others => '0');
		elsif rising_edge(clk) then

			if (LOADABLE and ld = '1') then
				m_count <= data;
			end if;

			if(CLOCK_ENABLE and en = '1') or (not CLOCK_ENABLE) then
				if(not DOWN_COUNTER) then
					m_count <= m_count + 1;
				else
					m_count <= m_count - 1;
				end if;
			end if;
         
		end if;
      
	end process;

	count <= m_count;

end behavioral;
