-------------------------------------------------------------------------------
-- Company    : ope
-- Author     : Olaf Petzold
--
-- Project    : Logic Analyzer
-- Title      : ram
-- File       : xc2s100_ram.vhd
-- Standard   : VHDL'93
-- Create Date: 2005-10-02
-- Last update: 2005-10-04
-- Platform   : XS2S100 Xilinx
-- Description: 
--    This is part of the logic analyzer project. For
--    further informations see at http://www.mikrocontroller.net.
--   
--
--    clk: Clock, Each port is fully synchronous with independent clock pins.
--       All port input pins have setup time referenced to the port
--       clk pin. The data output bus has a clock-to-out time referenced
--       to the clk pin.
--    reset: The reset pin forces the data output bus latches to zero synchronously.
--       This does not affect the memory cells of the
--       RAM and does not disturb a write operation on the other
--       port.
--    en: The enable pin affects the read, write and reset functionality
--       of the port. Ports with an inactive enable pin keep the output
--       pins in the previous state and do not write data to the memory
--       cells.
--    AB: The address bus selects the memory cells for read or write.
--       The width of the port determines the required width of this
--       bus as shown in Table 11.
--    DB_I: The data in bus provides the new data value to be written
--       into the RAM.
--    DB_O: The data out bus reflects the contents of the memory cells
--       referenced by the address bus at the last active clock edge.
--       During a write operation, the data out bus reflects the data
--       in bus.
--    we: Activating the write enable pin allows the port to write to the
--       memory cells. When active, the contents of the data input
--       bus are written to the RAM at the address pointed to by the
--       address bus, and the new data also reflects on the data out
--       bus. When inactive, a read operation occurs and the contents
--       of the memory cells referenced by the address bus
--       reflect on the data out bus.
--
--    There are two architectures:
--       * RAMB4_S16_banking, using xilinx's RAMB4_S16 block ram. Bug inside?
--         Got a lot of warnings.
--       * infering, using an array. It's hold a BUG too: it's
--         using an output latch even if not necessary. Wrong synthese? Further
--         more 'reset' isn't used.
--       
-- Copyright  : Copyright (c) 2005 Olaf Petzold
-- Revision   : $Id: xc2s100_ram.vhd,v 1.1 2005/10/04 08:50:11 olaf Exp $
-------------------------------------------------------------------------------

library unisim;
use unisim.vcomponents.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity ram_256x16 is
   
   generic (
      RESET_ACTIVE   : std_logic := '1';
      BIT_WIDTH      : positive  := 16
   );
   port (
      clk            : in  std_logic;
      rst            : in  std_logic;
      en             : in  std_logic;
      ADDR           : in  std_logic_vector(7 downto 0);
      DI             : in  std_logic_vector(BIT_WIDTH-1 downto 0);
      DO             : out std_logic_vector(BIT_WIDTH-1 downto 0);
      we             : in  std_logic
   );

end entity ram_256x16;


architecture ram_256x16_obuf of ram_256x16 is
   
   signal DB_O       : std_logic_vector(BIT_WIDTH-1 downto 0);

begin
   
   ram_block : RAMB4_S16
      -- RAMB4_S16: Virtex/E, Spartan-II/IIE 256 x 16 Single-Port RAM
      -- Xilinx  HDL Language Template version 7.1i
      generic map (
         INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
         INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map (
         CLK            => clk,           -- Clock input
         RST            => rst,           -- Synchronous reset input
         EN             => en,            -- RAM enable input
         ADDR           => ADDR,          -- 8-bit address input
         DO             => DB_O,          -- 16-bit data output
         DI             => DI,            -- 16-bit data input
         WE             => we             -- RAM write enable input
      );

   db_obuf: process (DB_O, en, we) is
   begin
      if (en = '1') and (we = '0') then
         DO <= DB_O;
      else
         DO <= (others => 'Z'); 
      end if; 
   end process db_obuf;
   
end architecture;


-------------------------------------------------------------------------------


library unisim;
use unisim.vcomponents.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity ram_1536x16 is
   
   generic (
      RESET_ACTIVE   : std_logic := '1';
      BIT_WIDTH      : positive  := 16;
      BIT_DEPTH      : positive  := 1536
   );
   port (
      clk            : in  std_logic;
      reset          : in  std_logic;
      en             : in  std_logic;
      AB             : in  std_logic_vector(10 downto 0);
      DB_I           : in  std_logic_vector(BIT_WIDTH-1 downto 0);
      DB_O           : out std_logic_vector(BIT_WIDTH-1 downto 0);
      we             : in  std_logic
   );

end entity ram_1536x16;

-------------------------------------------------------------------------------

architecture banking of ram_1536x16 is

   alias ADDR        : std_logic_vector(7 downto 0) is AB(7 downto 0);
   alias CS          : std_logic_vector(2 downto 0) is AB(10 downto 8);

   signal clk_ram		: std_logic;
   signal sel        : std_logic_vector(5 downto 0);
   
begin  -- architecture banking

   clk_bufg : BUFG                      -- xilinx
      port map (
         I              => clk,
         O              => clk_ram
      );

   
   chip_sel: process (CS, en) is

      variable bank, en_v : std_logic_vector(5 downto 0);

   begin
      
     case CS is
        when "000"  => bank := "000001";  -- bank #0
        when "001"  => bank := "000010";  -- bank #1
        when "010"  => bank := "000100";  -- bank #2                   
        when "011"  => bank := "001000";  -- bank #3                  
        when "100"  => bank := "010000";  -- bank #4
        when "101"  => bank := "100000";  -- bank #5
        when others => bank := "000000";
     end case;

     case en is
        when '0'    => en_v := "000000";
        when '1'    => en_v := "111111";
        when others => null;
     end case;

     sel <= bank and en_v;
     
   end process;

   
   bank0 : entity work.ram_256x16
      port map (
         CLK            => clk_ram,       -- Clock input
         RST            => reset,         -- Synchronous reset input
         EN             => sel(0),        -- RAM enable input
         ADDR           => ADDR,          -- 8-bit address input
         DO             => DB_O,          -- 16-bit data output
         DI             => DB_I,          -- 16-bit data input
         WE             => we             -- RAM write enable input
      );


   bank1 : entity work.ram_256x16
      port map (
         CLK            => clk_ram,       -- Clock input
         RST            => reset,         -- Synchronous reset input
         EN             => sel(1),        -- RAM enable input
         ADDR           => ADDR,          -- 8-bit address input
         DO             => DB_O,          -- 16-bit data output
         DI             => DB_I,          -- 16-bit data input
         WE             => we             -- RAM write enable input
      );


   bank2 : entity work.ram_256x16
      port map (
         CLK            => clk_ram,       -- Clock input
         RST            => reset,         -- Synchronous reset input
         EN             => sel(2),        -- RAM enable input
         ADDR           => ADDR,          -- 8-bit address input
         DO             => DB_O,          -- 16-bit data output
         DI             => DB_I,          -- 16-bit data input
         WE             => we             -- RAM write enable input
      );


   bank3 : entity work.ram_256x16
      port map (
         CLK            => clk_ram,       -- Clock input
         RST            => reset,         -- Synchronous reset input
         EN             => sel(3),        -- RAM enable input
         ADDR           => ADDR,          -- 8-bit address input
         DO             => DB_O,          -- 16-bit data output
         DI             => DB_I,          -- 16-bit data input
         WE             => we             -- RAM write enable input
      );


   bank4 : entity work.ram_256x16
      port map (
         CLK            => clk_ram,       -- Clock input
         RST            => reset,         -- Synchronous reset input
         EN             => sel(4),        -- RAM enable input
         ADDR           => ADDR,          -- 8-bit address input
         DO             => DB_O,          -- 16-bit data output
         DI             => DB_I,          -- 16-bit data input
         WE             => we             -- RAM write enable input
      );


   bank5 : entity work.ram_256x16
      port map (
         CLK            => clk_ram,       -- Clock input
         RST            => reset,         -- Synchronous reset input
         EN             => sel(5),        -- RAM enable input
         ADDR           => ADDR,          -- 8-bit address input
         DO             => DB_O,          -- 16-bit data output
         DI             => DB_I,          -- 16-bit data input
         WE             => we             -- RAM write enable input
      );

end architecture banking;


-------------------------------------------------------------------------------


architecture infering of ram_1536x16 is

   type ram_t is array(BIT_DEPTH-1 downto 0)
      of std_logic_vector(BIT_WIDTH-1 downto 0);

   -- infer RAM
   signal blockram   : ram_t; 

begin 

   read_proc: process (clk)
   begin
      if rising_edge(clk) then
         if (en = '1') and (we = '0') then
            DB_O <= blockram(conv_integer(AB));
         end if;
      end if;
   end process;

   write_proc: process (clk)
   begin
      if rising_edge(clk) then
         if (en = '1') and (we = '1') then
            blockram(conv_integer(AB)) <= DB_I;
         end if;
      end if;
   end process;

end architecture infering;


